import os
from .utils import get_config


PACKAGE_PATH = os.path.abspath(os.path.dirname(__file__))
REPO_PATH = os.path.join(PACKAGE_PATH, os.pardir)
CONFIG_PATH = os.path.expanduser('~/.adiumsh') \
    if not os.environ.get('ADIUMSH_TEST') \
    else os.path.join(REPO_PATH, '.adiumsh')

default_config = get_config(CONFIG_PATH, 'default')
DEFAULT_ACCOUNT = \
    default_config.get('account', None) if default_config else None
DEFAULT_SERVICE = \
    default_config.get('service', None) if default_config else None
DEFAULT_BUDDY = \
    default_config.get('buddy', None) if default_config else None



# -*- coding: utf-8 -*-
import argparse
import sys
from .settings import (DEFAULT_SERVICE, DEFAULT_ACCOUNT, DEFAULT_BUDDY)


def parse_args():
    parser = argparse.ArgumentParser(usage='%(prog)s [options]',
                                     prog='adiumsh')
    subparsers = parser.add_subparsers()

    # send
    parser_send = subparsers.add_parser('send')

    parser_send.add_argument('-m', '--message',
                             help='message to send')
    account_help = 'account used to send the message (for alias)'
    parser_send.add_argument('-t', '--account',
                             help=account_help)
    parser_send.add_argument('-s', '--service',
                             help='service associated with the account')
    send_buddy_group = parser_send.add_mutually_exclusive_group()
    send_buddy_group.add_argument('-b', '--buddy',
                                  help='name of the target account')
    send_buddy_group.add_argument('-a', '--alias',
                                  help='alias of the target account')
    args = parser.parse_args()
    if not args.buddy and not args.alias:
        args.buddy = args.buddy or DEFAULT_BUDDY
        if not args.buddy:
            msg = 'Must specify either buddy or alias'
            raise parser.error(msg)

    if args.alias and not args.buddy:
        args.service = args.service or DEFAULT_SERVICE
        args.account = args.account or DEFAULT_ACCOUNT
        if not args.service or not args.account:
            msg = 'Must specify service and account when using alias'
            raise parser.error(msg)
    if not args.message:
        message = sys.stdin.read()
        message = message.strip()
        if not message:
            msg = 'Message cannot be empty'
            raise parser.error(msg)
        args.message = message
    else:
        if not args.message.strip():
            msg = 'Message cannot be empty'
            raise parser.error(msg)
    return args



import os
import psutil
try:
    import configparser
except ImportError:
    import ConfigParser as configparser


def is_process_running(process_name):
    names = [proc.name() for proc in psutil.process_iter()]
    return process_name in names


def get_process(process_name):
    for proc in psutil.process_iter():
        if proc.name() == process_name:
            return proc
    else:
        return None


def get_config(path, section):
    config = configparser.ConfigParser()
    if os.path.exists(path):
        config.read(path)
        if config.has_section(section):
            return dict(config.items(section))



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, absolute_import
import glob
import os
#import shlex
import subprocess
import time
from .utils import is_process_running
from .settings import PACKAGE_PATH
from .command import parse_args


class Adium(object):
    script_prefix = r'adium-'
    script_ext = r'.scpt'
    open_cmd = ['open', '-a', 'Adium']

    def __init__(self, buddy=None, account=None, service=None):
        """
        :param buddy: account name of the target user to chat with
        :param account: default account to use
        :param service: default service of the account
        """
        if not self.is_running:
            self.start()
            while not self.is_running:
                time.sleep(1)
            # Wait for Adium to load accounts
            time.sleep(1)
        self.buddy = buddy
        self.account = account
        self.service = service

    @property
    def is_running(self):
        """Whether the application is running"""
        return is_process_running('Adium')

    def start(self):
        """Start the application"""
        subprocess.Popen(self.open_cmd)

    def send_alias(self, message, alias, account=None, service=None):
        """Send a message to an alias"""
        if account is None and self.account is not None:
            account = self.account
        if service is None and self.service is not None:
            service = self.service
        name = self.get_name(alias, account, service)
        self.send(message, name)

    def send(self, message, name=None):
        """Send a message"""
        if name is None and self.buddy is not None:
            name = self.buddy
        self.call_script('send', [name, message])

    def call_script(self, suffix, args):
        """
        Call AppleScript adium-suffix.scpt
        :param suffix: a string as the suffix of adium AppleScript
        :param args: a list of strings as arguments pass to the script
        """
        script = self.script_prefix + suffix + self.script_ext
        script_path = os.path.join(PACKAGE_PATH, script)
        path = os.path.join(PACKAGE_PATH, '*' + self.script_ext)
        if script_path in glob.glob(path):
            try:
                p = subprocess.Popen([script_path] + args,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
                stdout, stderr = p.communicate()
                if p.returncode != 0:
                    raise ExecutionError(stderr)
                return stdout
            except OSError as e:
                raise e
        else:
            raise DoesNotExist(script + ' does not exist')

    def get_name(self, alias, account=None, service=None):
        """Get account name by alias"""
        if account is None and self.account is not None:
            account = self.account
        if service is None and self.service is not None:
            service = self.service
        name = self.call_script('name', [service, account, alias])
        if name:
            return name.rstrip(b'\n')
        else:
            raise DoesNotExist('This alias does not exist in your account')


class DoesNotExist(Exception):
    pass


class ExecutionError(Exception):
    pass


def main():
    args = parse_args()
    adium = Adium(buddy=args.buddy,
                  account=args.account,
                  service=args.service)
    if args.buddy:
        adium.send(args.message, args.buddy)
    else:
        adium.send_alias(args.message, args.alias)



from setuptools import setup, find_packages
from adiumsh import __version__


setup(
    name='adium-sh',
    version=__version__,
    description='Command-line wrapper of Adium',
    long_description=open('README.rst').read(),
    keywords='adium',
    author='Shichao An',
    author_email='shichao.an@nyu.edu',
    url='https://github.com/shichao-an/adiumsh',
    license='BSD',
    install_requires=['argparse', 'psutil'],
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    package_data={
        'adiumsh': ['*.scpt'],
    },
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'adiumsh = adiumsh.adiumsh:main',
        ],
    },
    classifiers=[
        "Environment :: Console",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
)
